#!/usr/bin/env python

from jira import JIRA
import json
import logging
import os
import re
import requests
import yaml

FORMAT = '%(asctime)-15s %(levelname)s %(message)s'
logging.basicConfig(format=FORMAT)
logger = logging.getLogger('jira-importer')
logger.setLevel('DEBUG')

CONFIG_PATH = os.environ.get(
    'ZABBIX_IMPORTER_CONFIG_PATH',
    '/etc/racks/importers/jira-importer.yaml')

settings = yaml.load(open(CONFIG_PATH, 'r'))

for instance in settings.get('instances', []):
    config = settings['instances'].get(instance)

    RE_MB_SERIAL = re.compile(
        config.get('motherboard_serial_regex'), re.IGNORECASE)
    RE_IPMI_FQDN = re.compile(
        config.get('ipmi_fqdn_regex'), re.IGNORECASE)
    RE_IPMI_USER = re.compile(
        config.get('ipmi_user_regex'), re.IGNORECASE)
    RE_IPMI_PASSWORD = re.compile(
        config.get('ipmi_password_regex'), re.IGNORECASE)
    RE_MAC_ADDRESS = re.compile(
        config.get('mac_address_regex'), re.IGNORECASE)
    RE_VM_CONSOLE = re.compile(
        config.get('vm_console_regex'), re.IGNORECASE)

    key_cert_data = None
    with open(config.get('rsa_private_key'), 'r') as key_cert_file:
        key_cert_data = key_cert_file.read()

    oauth_dict = {
        'access_token': config.get('access_token'),
        'access_token_secret': config.get('access_token_secret'),
        'consumer_key': config.get('consumer_key'),
        'key_cert': key_cert_data
    }

    http_headers = {
        'User-agent': config.get('user_agent', 'Racks Jira Importer/0.1'),
    }

    jira_headers = http_headers.copy()
    racks_headers = http_headers.copy()
    racks_headers.update({
        'X-Auth-Application': config.get('racks_application'),
        'X-Auth-Token': config.get('racks_auth_token'),
    })

    logger.info('Initialized')

    jira = JIRA(config.get('jira_url'), oauth=oauth_dict)

    jql = config.get('jql_filter')
    logger.info('Filtering issues with %s' % jql)

    jira_tickets = jira.search_issues(jql)

    for jira_ticket in jira_tickets:
        logger.info('Working on %s' % (jira_ticket.key))
        issue = jira.issue(jira_ticket.key)

        for comment in issue.fields.comment.comments:
            attributes = {}
            kvm_console = re.search(RE_VM_CONSOLE, comment.body)

            if kvm_console:
                r = requests.get('%s/api/v1/lookup.json?query=%s' % (
                    config.get('racks_url'),
                    kvm_console.group(3)),
                    headers=racks_headers)
                try:
                    object_name = json.loads(r.text)[0]['object_name']
                    attributes['name'] = object_name
                except IndexError:
                    continue
                except KeyError:
                    continue

                attributes['attribute_ipmi_host'] = kvm_console.group(1)
                attributes['attribute_jira_issue'] = jira_ticket.key
            else:
                serials = re.findall(RE_MB_SERIAL, comment.body)
                logger.debug('[%s] Serials found: %s' % (
                    jira_ticket.key, serials))
                mac_adresses = re.findall(RE_MAC_ADDRESS, comment.body)
                logger.debug('[%s] MAC addresses found: %s' % (
                    jira_ticket.key, mac_adresses))
                identifiers = serials+mac_adresses
                for identifier in identifiers:
                    # Looks like normal check since I can't remember an idiot
                    # uses serial numbers shorter 6 symbols
                    if len(identifier) < 12:
                        continue

                    try:
                        url = '%s/api/v1/lookup.json?query=%s' % (
                            config.get('racks_url'), identifier)
                    except TypeError:
                        continue

                    r = requests.get(url, headers=racks_headers)
                    lookup_res = json.loads(r.text)
                    object_name = None
                    try:
                        object_name = lookup_res[0]['object_name']
                        attributes['name'] = object_name
                    except IndexError:
                        continue
                    except KeyError:
                        continue

                    attributes['attribute_jira_issue'] = jira_ticket.key

            if attributes:
                logger.info('[%s] Updating: %s' % (
                    jira_ticket.key, attributes))
                update = requests.post(
                    '%s/api/v1/import.json' % config.get('racks_url'),
                    attributes, headers=racks_headers)
